%{
/*****************************************************************************
 *              D: a minimal compiler for educational purpose                *
 ***************************************************************************** 
 *                                                                           *
 * Copyright (c) 2009 Christophe Alias                                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the     *
 * Free Software Foundation; either version 2 of the License, or (at your    * 
 * option) any later version.                                                *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General  *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * Written by Christophe Alias, Christophe.Alias@ens-lyon.fr                 *
 *                                                                           *
 *****************************************************************************/

#include <string.h>	/* for strcmp, strdup & friends */
#include <stdio.h>	/* for strcmp, strdup & friends */
#include <stdlib.h>	/* for atoi() */

#include "parser.h"	/* (TK_) symbols from parser.y */

extern int line_number;	/* defined in parser.y */

%}

%x comment

%%
[ \t\r]			break;		/* ignore white space */
"\n" 			++line_number; 

"("                     return TK_LPAR;
")"                     return TK_RPAR;
","                     return TK_COMMA;
";"                     return TK_SEMI;

"true"                  return TK_TRUE;
"false"                 return TK_FALSE;
">"                     return TK_GREATER;
"=="                    return TK_EQUAL;
"!="                    return TK_NEQUAL;

"+"                     return TK_PLUS;
"-"                     return TK_MINUS;
"*"                     return TK_MUL;

"int"                   return TK_INT;

"="                     return TK_ASSIGN; 

"if"                    return TK_IF;
"then"                  return TK_THEN;
"else"                  return TK_ELSE;
"fi"                    return TK_FI;
"while"                 return TK_WHILE;
"do"                    return TK_DO;
"done"                  return TK_DONE;
"return"                return TK_RETURN;

"function"              return TK_FUNCTION;
"begin"                 return TK_BEGIN;
"end"                   return TK_END;

"//"                    BEGIN(comment); //
<comment>[^\n]          break;          // Comments handling
<comment>\n             BEGIN(INITIAL); //

[0-9]+                  { 
                          yylval.integer = atoi(yytext); 
                          return TK_NUM; 
                        }

[A-Za-z_]([A-Za-z0-9_])* {
                          yylval.id_string = strdup(yytext);
                          //cout << yylval.id_string << endl;
                          return TK_ID;
                        }

.			{
			  fprintf(stderr,
				"Illegal character with code %d on"
				"line #%d\n",
				*yytext, line_number);
			  exit(1);
			}
